searchData={"items":[{"type":"module","title":"Const","doc":"Constants and Enumerated Values for Elixir","ref":"Const.html"},{"type":"module","title":"Overview - Const","doc":"This module adds support for a `const` macro that exports single constants\nand an `enum` macro that exports enumerated constant values from a module.\nThese values can be used in guards, match expressions or within normal\nexpressions, as the macro takes care of expanding the reference to the\nconstant or enumerated value to its corresponding literal value or function\ncall, depending on the context where it was used.\n\nA module using `const` or `enum` macros can be defined in the following way:\n```elixir\ndefmodule Settings\n  use Const\n  import Bitwise, only: [bsl: 2]\n\n  @ar \"AR\"\n  @it \"IT\"\n  @us \"US\"\n\n  const version, do: \"1.0\"\n\n  const base_path, do: System.cwd()\n\n  const country_codes, do: [@ar, @it, @us]\n\n  enum country_code, do: [argentina: @ar, italy: @it, usa: @us]\n\n  enum color do\n    red   bsl(0xff, 16)\n    green bsl(0xff, 8)\n    blue  bsl(0xff, 0)\n  end\n\n  enum color_tuple do\n    red   {255, 0, 0}\n    green {0, 255, 0}\n    blue  {0, 0, 255}\n  end\nend\n```\n\nAs you can see, the constants can be assigned both literal values or\nexpressions that will be resolved at compile-time.","ref":"Const.html#module-overview"},{"type":"module","title":"Single Constants - Const","doc":"You can create single constant values by using the `const` macro with the\nfollowing syntax:\n\n    const  , do:  \n\ne.g.\n\n    const version, do: \"1.0\"\n\nThe macro invocation will create and export another macro with the name that\nwas set in the `const` declaration (e.g. `version/0`) and replace each\nreference to it with the value that was assigned to it (e.g. `\"1.0\"`).\n\nYou can use any expression that can be resolved at compile-time as the value\nfor the `const`.\n\nThe single constants can be accessed with a nomal function invocation:\n```elixir\nrequire Settings\nSettings.version\n```\n\nAs the reference to the `const` will be replaced by its literal value, you\ncan even use them in match expressions or in guards. e.g.\n```elixir\nrequire Settings\nSettings.version = \"1.0\"\n```","ref":"Const.html#module-single-constants"},{"type":"module","title":"Enumerated Values - Const","doc":"You can create enumerated values by using the `enum` macro with the compact\nsyntax:\n\n    enum  , do: [ :  ,  :  , ...]\n\nOr with the expanded syntax:\n\n    enum   do\n         \n         \n      [...]\n    end\n\ne.g.\n```elixir\nenum country_code, do: [argentina: \"AR\", italy: \"IT\", usa: \"US\"]\n```\n\nOr:\n```elixir\nenum country_code do\n  argentina \"AR\"\n  italy     \"IT\"\n  usa       \"US\"\nend\n```\n\nFor each `enum` instance, the macro will create the following additional macros\nand functions in the module where it was invoked:\n\n  1. Macro with the name that was assigned to the `enum`. This macro will\n     replace every reference to itself with its literal value (if it was called\n     with a literal atom as key or was referenced from a match expression) or\n     with a call to the fallback function.\n  2. Fallback function with a name formed by appending the string `_enum` to\n     the name of the `enum` (e.g. `country_code_enum/1`).\n  3. Function that will retrieve the key corresponding to a value in the `enum`.\n     If there are is more than one key with the same value, the first in the\n     `enum` will be used and the other ones will be disregarded.\n  4. Macro with a name formed by appending the string `_values` to the name of\n     the enum (e.g. `country_code_values/0`).\n  5. Fallback function with a name formed by appending the string `_enum_values`\n     to the name of the values (e.g. `country_code_enum_values/0`).\n\ne.g.\n```elixir\ndefmacro country_code(atom) :: String.t\ndef country_code_enum(atom) :: String.t\ndef from_country_code(String.t) :: atom\ndefmacro country_code_values() :: [String.t()]\ndef country_code_enum_values() :: [String.t()]\n```\n\nThe enumerated values can be accessed with a function call:\n```elixir\nrequire Settings\nSettings.color(:blue)\n```\n\nAnd can also be used in match expressions or guards:\n```elixir\nrequire Settings\nimport Settings\nvalue = \"AR\"\ncase value do\n  country_code(:argentina) ->\n    {:ok, \"Argentina\"}\n  country_code(:italy) ->\n    {:ok, \"Italy\"}\n  code when code == country_code(:usa) ->\n    {:ok, \"United States\"}\n  _ ->\n    {:error, {:must_be_one_of, country_codes()}}\nend\n```\n\nAs the expressions assigned to constants will be resolved at compile-time,\nthe previous function would be equivalent to the following one:\n```elixir\nvalue = \"AR\"\ncase value do\n  \"AR\" -> {:ok, \"Argentina\"}\n  \"IT\" -> {:ok, \"Italy\"}\n  code when code == \"US\" -> {:ok, \"United States\"}\n  _ -> {:error, {:must_be_one_of, [\"AR\", \"IT\", \"US\"]}}\nend\n```\n\nSometimes, when an `enum` is referenced in the code, the key to its value is\npassed as an expression that cannot be resolved at compile-time. In those\ncases the expression will be expanded to a function invocation instead of to\na literal value:\n```elixir\nrequire Settings\nkey = :green\nSettings.color_tuple(key)\n```\n\nThis works because the macro replaces the reference to itself with a call to\nthe fallback function. The name of the function is that of the `enum`\nwith the `_enum` string appended to it. For example, for an enum named\n`country` the function will be `country_enum/1`. You have to keep this in\nmind when you import the module where the `enum` was defined and restrict\nthe functions that are imported.","ref":"Const.html#module-enumerated-values"},{"type":"macro","title":"Const.const/2","doc":"Defines a macro representing a single constant value that is resolved at\ncompile-time and is exported from the module. The macro can be used both in\nmatch and normal expressions in place of the literal value that was assigned\nto the constant.\n\nThe syntax to define it looks like the following one:\n```elixir\nconst secret_key, do: \"AABBCCDDEEFF\"\n```\n\nWhere `secret_key` is the name that will be used to reference the constant\nand `\"AABBCCDDEEFF\"` is what will be returned by it.\n\nThe constant can be referenced by using the normal macro/function syntax.\ne.g. `Settings.secret_key()`.\n\nGiven that the macro will replace its invocation with the literal value, it\ncan be used in a match expressions like the following one:\n```elixir\nrequire Settings\nSettings.color(:blue) = 0xff\n```","ref":"Const.html#const/2"},{"type":"macro","title":"Const.enum/2","doc":"Defines a macro representing an enumerated value that is resolved at\ncompile-time and is exported from the module. The macro can be used both in\nmatch and normal expressions in place of the literal value that was assigned\nto each value.\n\nTwo syntaxes are supported, the compact and the expanded one, very much like\nmost constructs in Elixir. Here's a sample of what it looks like\"\n```elixir\ndefmodule Defs\n  use Const\n\n  enum stock, do: [apple: \"AAPL\", facebook: \"FB\", google: \"GOOG\"]\n\n  enum file_ext do\n    elixir \".ex\"\n    erlang \".erl\"\n    go     \".go\"\n    rust   \".rs\"\n  end\nend\n```\n\nThe enumerated values can be referenced by using the normal macro/function\nsyntax, passing the key as argument to the macro.\ne.g. 'Config.file_ext(:elixir)'.\n\nGiven that the macro will replace its invocation with the literal value, it\ncan be used in a match expressions like the following one:\n```elixir\nDefs.stock(:google) = \"GOOG\"\n```","ref":"Const.html#enum/2"},{"type":"function","title":"Const.unescape_var/1","doc":"Unescape a quoted variable name.","ref":"Const.html#unescape_var/1"},{"type":"exception","title":"Const.Error","doc":"","ref":"Const.Error.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}